@@ -15,7 +15,8 @@ class PackEnv < Environment
 
   def a( *args )
     args.flatten!
-    indir g(:dir), args
+    ret = indir( g(:dir), args )
+		ret.size == 1 ? ret[0] : ret
   end
 
   alias d indir
@@ -41,7 +42,7 @@ def packing_do( task, recvname = nil )
   end
   unless recv.respond_to? task then
     raise NameError,
-      "task #{_name2str task} is not implemented for #{recv.name}"
+      "task '#{_name2str task}' is not implemented for #{recv.name}"
   end
   recv.send task
 end
