@@ -1,6 +1,6 @@
 /* vi:set ts=4 sw=4:
 
-  strscan.c version 0.5.2
+  strscan.c version 0.5.4
 
     Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
 
@@ -70,7 +70,7 @@ strscan_s_new(argc, argv, klass)
 
     p = ALLOC_N(struct strscanner, 1);
     p->flags = 0U;
-    if (RTEST(dup_p)) {
+    if (rb_safe_level() >= 4 || RTEST(dup_p)) {
         p->flags |= DUP_FLAG;
         p->ptr = ALLOC_N(char, RSTRING(str)->len) ;
         memcpy(p->ptr, RSTRING(str)->ptr, sizeof(char) * RSTRING(str)->len);
@@ -120,15 +120,12 @@ strscan_do_scan(self, regex, ptrflag, strflag)
     int ptrflag, strflag;
 {
     struct strscanner *p;
-    int ret ;
+    int ret;
 
     Check_Type(regex, T_REGEXP);
     GET_SCANNER(self, p);
 
-    if (SCAN_FINISHED(p)) return Qnil;
-        
     p->flags &= ~MATCH_FLAG;
-
     ret = re_match(RREGEXP(regex)->ptr,
                    p->ptr,
                    p->pend - p->ptr,
@@ -138,14 +135,14 @@ strscan_do_scan(self, regex, ptrflag, strflag)
         rb_raise(ScanError, "regexp buffer overflow");
     }
     else if ( ret < 0 ) {
-        return Qnil ;
+        return Qnil;
     }
     else {
         size_t len = p->regi.end[0];
 
         p->flags |= MATCH_FLAG;
         if (ptrflag) {
-            p->pre = p->ptr ;
+            p->pre = p->ptr;
             p->ptr += len;
             if (strflag)
                 return rb_str_new(p->pre, len);
@@ -186,13 +183,20 @@ strscan_skip(self, reg)
 
 
 static VALUE
-strscan_lookahead(self, reg)
+strscan_check(self, reg)
     VALUE self, reg;
 {
     return strscan_do_scan(self, reg, 0, 1);
 }
 
 
+static VALUE
+strscan_scan_full(self, reg, s, f)
+{
+    return strscan_do_scan(self, reg, RTEST(s), RTEST(f));
+}
+
+
 static VALUE
 strscan_getch(self)
     VALUE self;
@@ -200,9 +204,11 @@ strscan_getch(self)
     struct strscanner *p;
 
     GET_SCANNER(self, p);
+    p->flags &= ~MATCH_FLAG;
     if (SCAN_FINISHED(p))
-        return Qnil ;
-    
+        return Qnil;
+
+    p->flags |= MATCH_FLAG;
     p->pre = p->ptr;
     p->ptr += mbclen(*p->ptr);
     if (p->ptr > p->pend)
@@ -393,7 +399,8 @@ Init_strscan()
     rb_define_method(StringScanner, "scan",        strscan_scan,        1);
     rb_define_method(StringScanner, "skip",        strscan_skip,        1);
     rb_define_method(StringScanner, "match?",      strscan_match_p,     1);
-    rb_define_method(StringScanner, "lookahead",   strscan_lookahead,   1);
+    rb_define_method(StringScanner, "check",       strscan_check,       1);
+    rb_define_method(StringScanner, "scan_full",   strscan_scan_full,   3);
     rb_define_method(StringScanner, "getch",       strscan_getch,       0);
     rb_define_method(StringScanner, "empty?",      strscan_empty_p,     0);
     rb_define_method(StringScanner, "rest?",       strscan_rest_p,      0);
